#!/usr/bin/env python3
from __future__ import annotations
import json
import os
from typing import Any
import subprocess
from .student_repo import StudentRepo
from .class_task import ClassTask

class Collect:

    @staticmethod
    def run_tko_collect(repo: StudentRepo) -> str:
        """
        Run the tko collect command in the student's repository.
        """
        folder = repo.folder
        rep = repo.tko_subfolder
        print("Loading info from " + folder)
        rep_folder = os.path.join(folder, rep)
        if not os.path.isdir(rep_folder):
            return ""
        result = subprocess.run(["tko", "rep", rep_folder, "collect", "--daily", "--resume", "--game", "--json"], capture_output=True, text=True)
        return result.stdout


    @staticmethod
    def collect_student_text_map(class_task: ClassTask) -> dict[str, Any]:
        repo_list = class_task.load_student_repo_list()
        student_text_map: dict[str, Any] = {}
        for repo in repo_list:
            text = Collect.run_tko_collect(repo)
            if text != "":
                student_text_map[repo.get_student_key()] = text
        return student_text_map

    @staticmethod
    def main(target: str) -> None:
        class_task = ClassTask(target).load_from_file()
        student_text_map = Collect.collect_student_text_map(class_task)
        collected_path = class_task.get_default_collected()
        os.makedirs(os.path.dirname(collected_path), exist_ok=True)
        with open(collected_path, "w", encoding="utf-8") as f:
            json.dump(student_text_map, f, indent=4, ensure_ascii=False)


class TaskInfo:
    rate_max: int = 100
    flow_max: int = 6
    edge_max: int = 5
    neat_max: int = 5
    cool_max: int = 5
    easy_max: int = 5

    rate_str: str = "rate"
    flow_str: str = "flow"
    edge_str: str = "edge"
    neat_str: str = "neat"
    cool_str: str = "cool"
    easy_str: str = "easy"

    def __init__(self):
        self.rate: int = 0
        self.flow: int = 0
        self.edge: int = 0
        self.neat: int = 0
        self.cool: int = 0
        self.easy: int = 0
    
    def set_rate(self, value: int):
        if 0 <= value <= TaskInfo.rate_max:
            self.rate = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.rate_str}:{value}")
    
    def set_flow(self, value: int):
        if 0 <= value <= TaskInfo.flow_max:
            self.flow = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.flow_str}:{value}")

    def set_edge(self, value: int):
        if 0 <= value <= TaskInfo.edge_max:
            self.edge = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.edge_str}:{value}")

    def set_neat(self, value: int):
        if 0 <= value <= TaskInfo.neat_max:
            self.neat = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.neat_str}:{value}")
        
    def set_cool(self, value: int):
        if 0 <= value <= TaskInfo.cool_max:
            self.cool = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.cool_str}:{value}")

    def set_easy(self, value: int):
        if 0 <= value <= TaskInfo.easy_max:
            self.easy = value
            return self
        else:
            raise ValueError(f"Invalid {TaskInfo.easy_str}:{value}")
        
    def get_rate(self) -> int:
        return self.rate
    def get_flow(self) -> int:
        return self.flow
    def get_edge(self) -> int:
        return self.edge
    def get_neat(self) -> int:
        return self.neat
    def get_cool(self) -> int:
        return self.cool
    def get_easy(self) -> int:
        return self.easy
    

    def load_from_kv(self, kv: dict[str, str]):
        if TaskInfo.rate_str in kv:
            self.rate = int(kv[TaskInfo.rate_str])
        if TaskInfo.flow_str in kv:
            self.flow = int(kv[TaskInfo.flow_str])
        if TaskInfo.edge_str in kv:
            self.edge = int(kv[TaskInfo.edge_str])
        if TaskInfo.neat_str in kv:
            self.neat = int(kv[TaskInfo.neat_str])
        if TaskInfo.cool_str in kv:
            self.cool = int(kv[TaskInfo.cool_str])
        if TaskInfo.easy_str in kv:
            self.easy = int(kv[TaskInfo.easy_str])

    def get_kv(self) -> dict[str, str]:
        kv: dict[str, str] = {TaskInfo.rate_str: str(self.rate), TaskInfo.flow_str: str(self.flow),
                              TaskInfo.edge_str: str(self.edge), TaskInfo.neat_str: str(self.neat),
                              TaskInfo.cool_str: str(self.cool), TaskInfo.easy_str: str(self.easy)}
        return kv
    
    def get_filled_kv(self) -> dict[str, str]:
        kv: dict[str, str] = {}
        if self.rate > 0:
            kv[TaskInfo.rate_str] = str(self.rate)
        if self.flow > 0:
            kv[TaskInfo.flow_str] = str(self.flow)
        if self.edge > 0:
            kv[TaskInfo.edge_str] = str(self.edge)
        if self.neat > 0:
            kv[TaskInfo.neat_str] = str(self.neat)
        if self.cool > 0:
            kv[TaskInfo.cool_str] = str(self.cool)
        if self.easy > 0:
            kv[TaskInfo.easy_str] = str(self.easy)

        return kv
    
    


import os

class RepPaths:
    CFG_FILE = "repository.yaml"
    OLD_HISTORY_FILE = "history.csv"
    TASK_LOG_FILE = "task_log.csv"
    DAILY_FILE = "daily.yaml"
    INDEX_FILE = "Readme.md"
    TRACK_FOLDER = "track"
    LOG_FOLDER = "log"
    CONFIG_FOLDER = ".tko"

    def __init__(self, rep_dir: str):
        self.root_folder = rep_dir

    @staticmethod
    def rec_search_for_repo(folder: str) -> str:
        abs_folder: str = os.path.abspath(folder)
        if os.path.exists(os.path.join(abs_folder, RepPaths.CONFIG_FOLDER, RepPaths.CFG_FILE)):
            return abs_folder
        new_folder = os.path.dirname(abs_folder)
        if new_folder == abs_folder: # não encontrou, não tem mais como subir
            return ""
        return RepPaths.rec_search_for_repo(new_folder)
    
    def get_track_folder(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER, RepPaths.TRACK_FOLDER))

    def get_log_folder(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER, RepPaths.LOG_FOLDER))

    def get_track_task_folder(self, label: str) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER, RepPaths.TRACK_FOLDER, label))

    def get_config_folder(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER))

    def get_config_file(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER, RepPaths.CFG_FILE))

    def get_config_backup_file(self) -> str:
        return self.get_config_file() + ".backup"
    
    def get_old_history_file(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.CONFIG_FOLDER, RepPaths.OLD_HISTORY_FILE))

    def get_default_readme_path(self) -> str:
        return os.path.abspath(os.path.join(self.root_folder, RepPaths.INDEX_FILE))

    def get_rep_dir(self) -> str:
        return os.path.abspath(self.root_folder)
    
    def has_local_config_file(self) -> bool:
        return os.path.exists(self.get_config_file())


from tko.settings.settings import Settings
from tko.settings.repository import Repository
from tko.play.play import Play
from tko.util.text import Text

class CmdOpen:
    def __init__(self, settings: Settings):
        self.settings = settings
        self.need_update = False
        self.rep: Repository | None = None
        self.folder = ""

    def set_need_update(self):
        self.need_update = True

    def load_folder(self, folder: str):
        self.folder = folder
        self.rep = Repository(folder)
        if not self.rep.paths.has_local_config_file():
            print(Text.format("{r}: O parâmetro para o comando {g} {y} deve a pasta onde você iniciou o repositório.", "Erro", "tko open", "<pasta>"))
            print(Text.format("{g}: Navegue ou passe o caminho até a pasta do repositório e tente novamente.", "Ação"))
            print(Text.format("{g}: Ou use {y} para criar um novo repositório.", "Ação","tko init --remote [fup|poo|ed]"))
            raise Warning(Text.format("{r}: {y} {}", "Erro:", folder, "não contém um repositório do tko"))
        self.rep.load_config().load_game()
        return self


    def execute(self):
        if self.rep is None:
            raise Warning("Repositório não encontrado")

        play = Play(self.settings, self.rep)
        if self.need_update:
            play.set_need_update()
        play.play()



from __future__ import annotations
from tko.mico.student_repo import StudentRepo
import json
import os

class ClassTask:
    def __init__(self, file_path: str = ""):
        self.file_path: str = os.path.abspath(file_path)  # path to the class task file
        self.task_name = os.path.basename(self.file_path)[:-5]
        self.class_dir: str = ""  # directory of the class task
        self.cache_dir: str = "." # cache dir folder to store collected data
        self.sub_dir: str = "."    # subdirectory for the class task, usually [ fup | ed | poo | . ]
        self.reference_dir: str | None = None  # directory for reference data, usually [ reference/fup | reference/ed | reference/poo | . ]

    def __str__(self):
        return f"{self.class_dir} {self.sub_dir}"

    def __repr__(self):
        return f"JsonTask({self.class_dir}, {self.sub_dir})"

    def load_from_dict(self, json_data: dict[str, str]):
        file_path_dir = os.path.dirname(self.file_path)
        self.class_dir = os.path.normpath(os.path.join(file_path_dir, json_data.get("class", self.class_dir)))
        self.cache_dir = os.path.normpath(os.path.join(file_path_dir, json_data.get("cache", self.cache_dir)))
        self.sub_dir = json_data.get("subdir", self.sub_dir)
        if "reference" in json_data:
            self.reference_dir = os.path.normpath(os.path.join(file_path_dir, json_data.get("reference", "")))

    def load_from_file(self) -> ClassTask:
        if not os.path.isfile(self.file_path):
            print(f"Class task file not found: {self.file_path}")
            exit(1)
        with open(self.file_path, "r", encoding="utf-8") as f:
            data = json.load(f)
            self.load_from_dict(data)
            return self

    def get_reference_rep(self) -> str | None:
        return self.reference_dir
    
    def get_default_collected(self) -> str:
        return os.path.join(self.cache_dir, f"{self.task_name}_collected.json")

    def get_default_graph_joined(self) -> tuple[str, str]:
        name = os.path.join(self.cache_dir, f"{self.task_name}_graph_")
        return name + "color.txt", name + "mono.txt"

    def get_default_simple_csv(self) -> str:
        return os.path.join(self.cache_dir, f"{self.task_name}_grade_tasks.csv")
    
    def get_default_quest_csv(self) -> str:
        return os.path.join(self.cache_dir, f"{self.task_name}_grade_quests.csv")

    def get_default_full_csv(self) -> str:
        return os.path.join(self.cache_dir, f"{self.task_name}_grade_full.csv")

    def load_student_repo_list(self) -> list[StudentRepo]:
        student_list: list[str] = os.listdir(self.class_dir)
        student_list = sorted(student_list, key=lambda x: x.lower())  # sort students by name
        student_folder_list = [StudentRepo(os.path.join(self.class_dir, student), self.sub_dir) for student in student_list]
        return student_folder_list


import os

class StudentRepo:
    def __init__(self, folder: str, rep: str):
        self.__student_key: str = os.path.basename(folder)
        self.folder = folder
        self.tko_subfolder = rep

    def get_student_key(self) -> str:
        return self.__student_key

    def __str__(self):
        return f"{self.folder} {self.tko_subfolder}"

    def __repr__(self):
        return f"Target({self.folder}, {self.tko_subfolder})"
    



#!/usr/bin/env python3
from __future__ import annotations
import argparse
from .collect import Collect
from .pull import Pull
from .join import Join, update_reference_rep
from icecream import ic

def pull(args: argparse.Namespace) -> None:
    for target in args.targets:
        Pull.main(target, args.threads)

def collect(args: argparse.Namespace) -> None:
    for target in args.targets:
        Collect.main(target)

def join(args: argparse.Namespace) -> None:
    for target in args.targets:
        Join.main(target)

def refupdate(args: argparse.Namespace) -> None:
    for target in args.targets:
        update_reference_rep(target)

def combined(args: argparse.Namespace) -> None:
    
    if args.pull:
        for target in args.targets:
            Pull.main(target)
    if args.refupdate:
        for target in args.targets:
            update_reference_rep(target)
    if args.collect:
        for target in args.targets:
            Collect.main(target)
    if args.join:
        for target in args.targets:
            Join.main(target)

def mico_main():
    # Caminho do arquivo CSV
    parser = argparse.ArgumentParser(description="Ferramenta para lidar com tarefas do classroom usando o tko.")
    parser.add_argument("-d", "--debug", action="store_true", help="Enable debug mode.")
    subcommands = parser.add_subparsers(dest="command", required=True)

    pull_cmd = subcommands.add_parser("pull", help="Pull data from the specified target.")
    pull_cmd.add_argument("targets", type=str, nargs="+", help="Json class task file")
    pull_cmd.add_argument("-t", "--threads", type=int, default=5, help="Number of threads to use for pulling data.")
    pull_cmd.set_defaults(func=pull)

    # Comando para atualizar repositório de referência
    refupdate_cmd = subcommands.add_parser("refupdate", help="Update the reference repository.")
    refupdate_cmd.add_argument("targets", type=str, nargs="+", help="Json class task file")
    refupdate_cmd.set_defaults(func=refupdate)

    collect_cmd = subcommands.add_parser("collect", help="Collect data from the specified target.")
    collect_cmd.add_argument("targets", type=str, nargs="+", help="Json class task files")
    collect_cmd.set_defaults(func=collect)

    join_cmd = subcommands.add_parser("join", help="Join collected data from the specified target.")
    join_cmd.add_argument("targets", type=str, nargs="+", help="Json class task files")
    join_cmd.set_defaults(func=join)

    combined_cmd = subcommands.add_parser("combi", help="Run many commands in many tasks.")
    combined_cmd.add_argument("targets", type=str, nargs="+", help="Json class task files")
    combined_cmd.add_argument("--refupdate", "-r", action="store_true", help="Update the reference repository.")
    combined_cmd.add_argument("--pull", "-p", action="store_true", help="Pull data from the specified targets.")
    combined_cmd.add_argument("--collect", "-c", action="store_true", help="Collect data from the specified targets.")
    combined_cmd.add_argument("--join", "-j", action="store_true", help="Join collected data from the specified targets.")
    combined_cmd.set_defaults(func=combined)

    args = parser.parse_args()
    if not args.debug:
        ic.disable()
    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    mico_main()



from __future__ import annotations
from tko.settings.task_listener import TaskListener
from tko.settings.week_listener import WeekListener
from tko.logger.log_history import LogHistory
from tko.logger.log_item_base import LogItemBase

class Logger:

    def __init__(self, rep_folder: str):
        self.tasks = TaskListener()
        self.week = WeekListener()
        self.history = LogHistory(rep_folder, [self.tasks.handle_log_entry, self.week.handle_entry_incoming])
    
    def store(self, action: LogItemBase):
        self.history.append_new_action(action)
